/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): Marc Herbert
 */
#ifndef SYSTEMDEPENDANTDEFS_HPP_
#define SYSTEMDEPENDANTDEFS_HPP_

// http://predef.sourceforge.net/ is interesting but quite outdated


////////////////////////////////////////////////////////////////////////////////
// Sockets options
/**
 * In Carob, we want to recover from closed socket errors. So we will tell the
 * system *not* to send sigpipes when writing on a closed socket.
 * Solutions to this are system dependant:
 * -> *nixes have MSG_NOSIGNAL flag for send(2/3) function
 * -> MaxOS/X >= 10.1 and old FreeBSD have no MSG_NOSIGNAL but a socket option SO_NOSIGPIPE
 *    (check socket(7) and setsockopt(2))
 *    http://developer.apple.com/documentation/Darwin/Reference/ManPages/man2/setsockopt.2.html
 * -> ...
 */

#ifdef __MINGW32__
#  include <ws2tcpip.h>
#else
#  include <sys/types.h>
#  include <sys/socket.h>
#endif


#if !defined(MSG_NOSIGNAL)

#  if defined(SO_NOSIGPIPE)
     // If we can rely on setsockopt(SO_NOSIGPIPE)
     // then we safely provide a dummmy MSG_NOSIGNAL
#    define MSG_NOSIGNAL 0

#  elif defined(__MINGW32__)
     // NOP! MINGW32 is OK (really?)

#  else
#    error "Could not find either MSG_NOSIGNAL nor SO_NOSIGPIPE"
     // Solaris provides neither and DOES sends a SIGPIPE (see setsockopt(3SOCKET)
     // We have a problem here... See tentative solutions in CAROB-116

#  endif

#endif


namespace
{

/** Write options. Flag given for each send() on the socket */
const int SOCKET_SEND_FLAGS               = MSG_NOSIGNAL;

}; //anonymous namespace
#endif /*SYSTEMDEPENDANTDEFS_HPP_*/
