/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005 Emic Networks
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#ifndef TYPETAG_H_
#define TYPETAG_H_

#include <string>

#include "DriverSocket.hpp"

//TODO: This file should be generated by parsing the java source
// code "TypeTag.java" in order to be sure to stick to the java definitions.
// For that, the java source should be re-formated to allow easy (perl?) 
// parsing of the file
#include "TypeConstants.hpp"

namespace CarobNS {

/**
 * This class implements protocol type tags with an internal String, but offers
 * an interface on top of it in order to be transparently substituted
 * some day (with enums for instance).
 * <p>
 * Advantages of using string types is human-readability (debugging, trace
 * analysis, etc.) and earlier detection in case of protocol corruption.
 * Drawback maybe a small performance cost.
 */
class TypeTag
{
public:
  /**
   * Constructor with the typetag number
   * @param tn type number to affect
   * @see TypeTagConstants.hpp for possible values
   */
  TypeTag(int tn) : typeNumber(tn) {};
  
  /**
   * Reads/deserializes/constructs a TypeTag from a stream.
   * 
   * @param in input driver socket
   * @throws SocketIOException stream error
   */
  TypeTag(const DriverSocket& in) throw (SocketIOException, UnexpectedException)
      { in>>typeNumber; }

  /**
   * Serialize "this" tag on the stream.
   * 
   * @param out output stream
   * @throws SocketIOException stream error
   */
  void sendToStream(const DriverSocket& out) const throw (SocketIOException,
      UnexpectedException) { out<<typeNumber; }

  /**
   * Comparison with an integer: compares typeNumber
   */
  bool operator == (const int& ref) { return typeNumber == ref; }

  /**
   * Cast to integer
   */  
  operator int () { return typeNumber; }
private:
  /** Actual Type */
  int typeNumber;
};

} //namespace CarobNS
#endif /*TYPETAG_H_*/
