/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "ResultSetMetaData.hpp"

#include "Field.hpp"
#include "Common.hpp"

#include "DriverResultSet.hpp"

using std::wstring;

using namespace CarobNS;

ResultSetMetaData::ResultSetMetaData(DriverResultSet* rs)
    throw (DriverException, UnexpectedException)
{
  if (rs == NULL)
    throw DriverException(L"Null resultSet, cannot get column count");
  resultSetPtr = rs;
}
int ResultSetMetaData::getColumnCount()
{
  return resultSetPtr->nbOfColumns;
}
void ResultSetMetaData::checkColumnIndex(int column) throw (DriverException,
    UnexpectedException)
{
  if ((column < 1) || (column > resultSetPtr->nbOfColumns))
    throw DriverException(L"Invalid column index " + toUserString(column)
        + L" is not between 1 and " + toUserString(resultSetPtr->nbOfColumns));
}
bool ResultSetMetaData::isAutoIncrement(int column) throw (DriverException,
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isAutoIncrement();
}
bool ResultSetMetaData::isCaseSensitive(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isCaseSensitive();
}
bool ResultSetMetaData::isSearchable(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isSearchable();
}
bool ResultSetMetaData::isCurrency(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isCurrency();
}
int ResultSetMetaData::isNullable(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isNullable();
}
bool ResultSetMetaData::isSigned(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isSigned();
}
int ResultSetMetaData::getColumnDisplaySize(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getColumnDisplaySize();
}
wstring ResultSetMetaData::getColumnLabel(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getFieldLabel();
}
wstring ResultSetMetaData::getColumnName(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getFieldName();
}
wstring ResultSetMetaData::getSchemaName(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getFullName();
}
int ResultSetMetaData::getPrecision(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getPrecision();
}
int ResultSetMetaData::getScale(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getScale();
}
wstring ResultSetMetaData::getTableName(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getTableName();
}
wstring ResultSetMetaData::getCatalogName(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return (L"");
}
int ResultSetMetaData::getColumnType(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getSqlType();
}
wstring ResultSetMetaData::getColumnTypeName(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getTypeName();
}
bool ResultSetMetaData::isReadOnly(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isReadOnly();
}
bool ResultSetMetaData::isWritable(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isWritable();
}
bool ResultSetMetaData::isDefinitelyWritable(int column) throw (DriverException, 
    UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->isDefinitelyWritable();
}
wstring ResultSetMetaData::getColumnClassName(int column)
    throw (DriverException, UnexpectedException)
{
  checkColumnIndex(column);
  return resultSetPtr->fields[column - 1]->getColumnClassName();
}
